@@ -276,7 +276,7 @@ paint_button (cairo_t *cr, GtkStyle *style, GtkStateType state_type, GtkShadowTy
 			/* inside shadow */
 			radius = 1.0;
 			cairo_move_to (cr, x + 1.0, y + height - 1.0);
-			cairo_arc (cr, x + radius + 1.0, y + radius + 1.0, radius, M_PI, M_PI * 1.5);
+			cairo_arc (cr, x + radius + 1.0, y + radius + 1.0, radius, G_PI, G_PI * 1.5);
 			cairo_line_to (cr, x + width - 1.0, y + 1.0);
 			cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.22);
 			cairo_stroke (cr);
@@ -346,11 +346,10 @@ paint_shadow (cairo_t *cr, GtkStyle *style,
 	width -= 1.0; height -= 1.0;
 
 	/* outer rectangle */
-	cairo_rectangle (cr, x, y, width, height);
 	if (shadow_type == GTK_SHADOW_OUT || shadow_type == GTK_SHADOW_ETCHED_OUT)
 	{
 		cairo_set_source_rgb (cr, OUTLINE_GRAY);
-		cairo_stroke (cr);
+		ge_cairo_stroke_rectangle (cr, x, y, width, height);
 	}
 	else if (shadow_type == GTK_SHADOW_IN || shadow_type == GTK_SHADOW_ETCHED_IN)
 	{
@@ -362,7 +361,7 @@ paint_shadow (cairo_t *cr, GtkStyle *style,
 		cairo_pattern_add_color_stop_rgba (crp, 1.0, 1.0, 1.0, 1.0, 0.5);
 		cairo_pattern_add_color_stop_rgba (crp, 0.0, .0, .0, .0, 0.2);
 		cairo_set_source (cr, crp);
-		cairo_stroke (cr);
+		ge_cairo_stroke_rectangle (cr, x, y, width, height);
 		cairo_pattern_destroy (crp);
 	}
 
@@ -370,8 +369,6 @@ paint_shadow (cairo_t *cr, GtkStyle *style,
 	x += 1.0; y += 1.0;
 	width -= 2.0; height -= 2.0;
 
-	cairo_rectangle (cr, x, y, width, height);
-
 	/* stroke */
 	if (shadow_type == GTK_SHADOW_OUT || shadow_type == GTK_SHADOW_ETCHED_OUT)
 	{
@@ -383,14 +380,14 @@ paint_shadow (cairo_t *cr, GtkStyle *style,
 		cairo_pattern_add_color_stop_rgba (crp, 0.0, 1.0, 1.0, 1.0, 0.5);
 		cairo_pattern_add_color_stop_rgba (crp, 1.0, .0, .0, .0, 0.2);
 		cairo_set_source (cr, crp);
-		cairo_stroke (cr);
+		ge_cairo_stroke_rectangle (cr, x, y, width, height);
 		cairo_pattern_destroy (crp);
 	}
 	else if (shadow_type == GTK_SHADOW_IN || shadow_type == GTK_SHADOW_ETCHED_IN)
 	{
 		/*TODO: Find a way to calculate this value (same as shadow out outer line)*/
 		cairo_set_source_rgb (cr, OUTLINE_GRAY);
-		cairo_stroke (cr);
+		ge_cairo_stroke_rectangle (cr, x, y, width, height);
 
 		/*
 		// three lines of alpha: 0.22, 0.12, 0.03
@@ -430,11 +427,10 @@ paint_entry_shadow (cairo_t *cr, GtkStyle *style,
 
 
 	/* Outer Line */
-	cairo_rectangle (cr, x, y, width, height);
 	if (focused)
 	{
 		gdk_cairo_set_source_color (cr, &style->light[GTK_STATE_SELECTED]);
-		cairo_stroke (cr);
+		ge_cairo_stroke_rectangle (cr, x, y, width, height);
 	}
 	else
 	{
@@ -443,22 +439,24 @@ paint_entry_shadow (cairo_t *cr, GtkStyle *style,
 		cairo_pattern_add_color_stop_rgb (crp, 0.0, 185/255.0, 189/255.0, 182/255.0);
 		cairo_pattern_add_color_stop_rgb (crp, 1.0, 238/255.0, 238/255.0, 236/255.0);
 		cairo_set_source (cr, crp);
-		cairo_stroke (cr);
+		ge_cairo_stroke_rectangle (cr, x, y, width, height);
 		cairo_pattern_destroy (crp);
 	}
 
 	/* inner Line */
-	cairo_rectangle (cr, x + 1.0, y + 1.0, width - 2.0, height - 2.0);
 	/* TODO: Find a way to calculate these values */
-	cairo_set_source_rgb (cr, 54/255.0, 52/255.0, 54/255.0);
-	cairo_stroke (cr);
+	if (state_type == GTK_STATE_INSENSITIVE)
+		gdk_cairo_set_source_color (cr, &style->fg [state_type]);
+	else
+		cairo_set_source_rgb (cr, 54/255.0, 52/255.0, 54/255.0);
+
+	ge_cairo_stroke_rectangle (cr, x + 1.0, y + 1.0, width - 2.0, height - 2.0);
 
 	/* inner shadow */
 
 	/* make sure background is initialised as shadow is only in top left */
 	gdk_cairo_set_source_color (cr, &style->base[(state_type == GTK_STATE_INSENSITIVE) ? GTK_STATE_INSENSITIVE : GTK_STATE_NORMAL]);
-	cairo_rectangle (cr, x + 2.0, y + 2.0, width - 4.0, height - 4.0);
-	cairo_stroke (cr);
+	ge_cairo_stroke_rectangle (cr, x + 2.0, y + 2.0, width - 4.0, height - 4.0);
 
 	/* draw shadow */
 	cairo_move_to (cr, x + 2.0, y + height - 2.0);
@@ -481,7 +479,7 @@ paint_scrollbar_trough (cairo_t *cr, GtkStyle *style, GtkStateType state_type, G
 
 	#define OFFWHITE 238/255.0, 238/255.0, 238/255.0
 
-
+	/* bevel */
 	ge_gdk_color_to_cairo (&style->white, &br);
 	ge_gdk_color_to_cairo (&style->bg[state_type], &tl);
 	ge_shade_color (&tl, 1.2, &tl);
@@ -496,9 +494,9 @@ paint_scrollbar_trough (cairo_t *cr, GtkStyle *style, GtkStateType state_type, G
 	cairo_fill (cr);
 
 	if (orientation == GTK_ORIENTATION_HORIZONTAL)
-		cairo_rectangle (cr, x, y, width, height * 0.4);
+		cairo_rectangle (cr, x, y, width, height / 2 - 1);
 	else
-		cairo_rectangle (cr, x, y, width * 0.4, height);
+		cairo_rectangle (cr, x, y, width /2 - 1, height);
 
 	cairo_set_source_rgb (cr, OFFWHITE);
 	cairo_fill_preserve (cr);
@@ -508,9 +506,9 @@ paint_scrollbar_trough (cairo_t *cr, GtkStyle *style, GtkStateType state_type, G
 
 
 	if (orientation == GTK_ORIENTATION_HORIZONTAL)
-		cairo_rectangle (cr, x, y + height * 0.6, width, height * 0.4);
+		cairo_rectangle (cr, x, y + height / 2 + 1, width, height  / 2 -1);
 	else
-		cairo_rectangle (cr, x + width * 0.6, y, width * 0.4, height);
+		cairo_rectangle (cr, x + width / 2 + 1, y, width / 2 - 1, height);
 
 	cairo_set_source_rgb (cr, OFFWHITE);
 	cairo_fill_preserve (cr);
@@ -530,20 +528,86 @@ paint_scrollbar_trough (cairo_t *cr, GtkStyle *style, GtkStateType state_type, G
 	}
 
 
+	/* set co-ordinates for fill drawing */
+	x -= 0.5; y -= 0.5; width += 1.0; height += 1.0;
+
 	cairo_rectangle (cr, x, y, width, height);
-	cairo_pattern_add_color_stop_rgba (crp, 0.0, 0.0, 0.0, 0.0, 0.4);
+	cairo_pattern_add_color_stop_rgba (crp, 0.0, 0.0, 0.0, 0.0, 0.5);
 	cairo_pattern_add_color_stop_rgba (crp, gradient_size, 0.0, 0.0, 0.0, 0.0);
-	cairo_pattern_add_color_stop_rgba (crp, 1 - gradient_size, 0.0, 0.0, 0.0, 0.0);
-	cairo_pattern_add_color_stop_rgba (crp, 1.0, 0.0, 0.0, 0.0, 0.4);
+	cairo_pattern_add_color_stop_rgba (crp, 1.0 - gradient_size, 0.0, 0.0, 0.0, 0.0);
+	cairo_pattern_add_color_stop_rgba (crp, 1.0, 0.0, 0.0, 0.0, 0.5);
 
 	cairo_set_source (cr, crp);
 	cairo_fill (cr);
 	cairo_pattern_destroy (crp);
 }
 
+static void
+paint_scale_trough (cairo_t *cr, GtkStyle *style, GtkStateType state_type, GtkOrientation orientation,
+	gdouble x, gdouble y, gdouble width, gdouble height)
+{
+	CairoColor c1, c2;
+	cairo_pattern_t *crp;
+	gdouble radius = 2.0;
+
+	/* set co-ordinates for line drawing */
+	x += 0.5; y += 0.5; width -= 1.0; height -= 1.0;
+
+
+	if (orientation == GTK_ORIENTATION_VERTICAL)
+		crp = cairo_pattern_create_linear (x + 2.0, y, x + width - 2.0, y);
+	else
+		crp = cairo_pattern_create_linear (x, y+2.0, x, y+height-2.0);
+
+	ge_cairo_rounded_rectangle (cr, x, y, width, height, radius, CR_CORNER_ALL);
+
+	/* calculate start and end colours for the gradient */
+	ge_gdk_color_to_cairo (&style->bg[GTK_STATE_NORMAL], &c1);
+	ge_shade_color (&c1, 0.7, &c2); /* darken */
+	ge_shade_color (&c1, 1.1, &c1); /* lighten */
+
+
+	cairo_pattern_add_color_stop_rgb (crp, 0.0, c1.r, c1.g, c1.b);
+	cairo_pattern_add_color_stop_rgb (crp, 1.0, c2.r, c2.g, c2.b);
+
+	cairo_set_source (cr, crp);
+	cairo_fill_preserve (cr);
+	cairo_pattern_destroy (crp);
+
+	/* dark gray */
+
+	cairo_set_source_rgb (cr, OUTLINE_GRAY);
+	cairo_stroke (cr);
+
+	ge_cairo_rounded_rectangle (cr, x+1.0, y+1.0, width-2.0, height-2.0, radius, CR_CORNER_ALL);
+	crp = cairo_pattern_create_linear (x, y, x, y + height);
+	cairo_pattern_add_color_stop_rgb (crp, 0.0, 1.0, 1.0, 1.0);
+	cairo_pattern_add_color_stop_rgba (crp, 1.0, 1.0, 1.0, 1.0, 100/255.0);
+	cairo_set_source (cr, crp);
+	cairo_stroke (cr);
+
+	ge_gdk_color_to_cairo (&style->bg[GTK_STATE_NORMAL], &c2);
+	ge_shade_color (&c2, 0.8, &c2); /* darken */
+	cairo_set_source_rgb (cr, c2.r, c2.g, c2.b);
+	if (orientation == GTK_ORIENTATION_VERTICAL)
+	{
+		cairo_move_to (cr, x + width + 1.0, y + radius);
+		cairo_line_to (cr, x + width + 1.0, y + height - radius);
+	}
+	else
+	{
+		cairo_move_to (cr, x + radius, y + height + 1.0);
+		cairo_line_to (cr, x + width - radius, y + height + 1.0);
+	}
+	cairo_stroke (cr);
+
+	cairo_pattern_destroy (crp);
+
+}
+
 static void
 paint_progress_bar (cairo_t *cr, GtkStyle *style, GtkStateType state_type, GtkProgressBarOrientation orientation,
-		    gdouble x, gdouble y, gdouble width, gdouble height)
+		    gdouble x, gdouble y, gdouble width, gdouble height, gdouble fraction)
 {
 	cairo_pattern_t *crp;
 	CairoColor c1, c2;
@@ -568,6 +632,19 @@ paint_progress_bar (cairo_t *cr, GtkStyle *style, GtkStateType state_type, GtkPr
 	/* set co-ordinates for line drawing */
 	x += 0.5; y += 0.5; width -= 1.0; height -= 1.0;
 
+	if (fraction < 1.0)
+	{
+		switch (orientation)
+		{
+			case GTK_PROGRESS_LEFT_TO_RIGHT: width--; break;
+			case GTK_PROGRESS_RIGHT_TO_LEFT: x++; width--; break;
+			case GTK_PROGRESS_TOP_TO_BOTTOM: height--; break;
+			case GTK_PROGRESS_BOTTOM_TO_TOP: y++; height--; break;
+		}
+	}
+
+
+
 	/* bevel */
 	cairo_rectangle (cr, x, y, width, height);
 	crp = cairo_pattern_create_linear (x, y, x, y + height);
@@ -581,31 +658,32 @@ paint_progress_bar (cairo_t *cr, GtkStyle *style, GtkStateType state_type, GtkPr
 
 	/* end cap */
 
-	cairo_set_source_rgb (cr, c2.r, c2.g, c2.b);
-	switch (orientation)
+	if (fraction < 1.0)
 	{
-		case GTK_PROGRESS_LEFT_TO_RIGHT:
-			cairo_move_to (cr, x + width + 1.0, y);
-			cairo_line_to (cr, x + width + 1.0, y + height);
-			break;
-		case GTK_PROGRESS_RIGHT_TO_LEFT:
-			cairo_move_to (cr, x - 1.0, y);
-			cairo_line_to (cr, x - 1.0, y + height);
-			break;
-		case GTK_PROGRESS_TOP_TO_BOTTOM:
-			cairo_move_to (cr, x, y + height + 1.0);
-			cairo_line_to (cr, x + width, y + height + 1.0);
-			break;
-		case GTK_PROGRESS_BOTTOM_TO_TOP:
-			cairo_move_to (cr, x, y - 1.0);
-			cairo_line_to (cr, x + width, y - 1.0);
-			break;
+		cairo_set_source_rgb (cr, c2.r, c2.g, c2.b);
+		switch (orientation)
+		{
+			case GTK_PROGRESS_LEFT_TO_RIGHT:
+				cairo_move_to (cr, x + width + 1.0, y);
+				cairo_line_to (cr, x + width + 1.0, y + height);
+				break;
+			case GTK_PROGRESS_RIGHT_TO_LEFT:
+				cairo_move_to (cr, x - 1.0, y);
+				cairo_line_to (cr, x - 1.0, y + height);
+				break;
+			case GTK_PROGRESS_TOP_TO_BOTTOM:
+				cairo_move_to (cr, x, y + height + 1.0);
+				cairo_line_to (cr, x + width, y + height + 1.0);
+				break;
+			case GTK_PROGRESS_BOTTOM_TO_TOP:
+				cairo_move_to (cr, x, y - 1.0);
+				cairo_line_to (cr, x + width, y - 1.0);
+				break;
+		}
+		cairo_stroke (cr);
 	}
 
 
-	cairo_stroke (cr);
-
-
 	/* draw the end cap shadow
 	 * TODO: create shadows for all progress bars (currently only works for left to right)
 	 *
@@ -761,7 +839,7 @@ draw_shadow (GtkStyle *style,
 	if (widget != NULL && GTK_WIDGET_HAS_FOCUS (widget))
 		focused = TRUE;
 
-	if (widget && (IS_COMBO (widget->parent) || IS_COMBO_BOX_ENTRY (widget->parent)))
+	if (widget && (GE_IS_COMBO (widget->parent) || GE_IS_COMBO_BOX_ENTRY (widget->parent)))
 	{
 		GtkWidget *button;
 		if (ge_widget_is_ltr (widget))
@@ -776,7 +854,7 @@ draw_shadow (GtkStyle *style,
 			area = &area2;
 		g_object_set_data ((GObject*) widget->parent, "entry", widget);
 		button = g_object_get_data ((GObject*) widget->parent, "button");
-		if (GTK_IS_WIDGET (button))
+		if (GE_IS_WIDGET (button))
 			gtk_widget_queue_draw_area (button,  button->allocation.x, button->allocation.y, button->allocation.width,button->allocation.height);
 	}
 	cr = ge_gdk_drawable_to_cairo (window, area);
@@ -810,6 +888,7 @@ draw_box (GtkStyle *style,
 	if (DETAIL ("spinbutton"))
 	{
 		/* this is a border round spin buttons - we don't want anything here */
+		gtk_style_apply_default_background (style, window, TRUE, state_type, area, x, y, width, height);
 		return;
 	}
 
@@ -829,9 +908,13 @@ draw_box (GtkStyle *style,
 				height--;
 			else
 				height -= 2;
+
+			if (state_type == GTK_STATE_INSENSITIVE)
+				height++;
+
 		}
-		
-		if (widget && (IS_COMBO (widget->parent) || IS_COMBO_BOX_ENTRY (widget->parent)))
+
+		if (widget && (GE_IS_COMBO (widget->parent) || GE_IS_COMBO_BOX_ENTRY (widget->parent)))
 		{
 			/* Combobox buttons */
 			gboolean focused = FALSE;
@@ -858,11 +941,13 @@ draw_box (GtkStyle *style,
 			else
 				paint_entry_shadow (cr, style, state_type, focused, x, y, width + 4, height);
 
+			if (state_type == GTK_STATE_INSENSITIVE)
+				height++;
 
 			x += 3; y += 3;
 			width -= 6; height -= 6;
 		}
-		if (widget && (GTK_IS_TREE_VIEW (widget->parent)))
+		if (widget && (GE_IS_TREE_VIEW (widget->parent)))
 		{
 			/* Add some extra padding for treeview column buttons */
 			if (state_type == GTK_STATE_INSENSITIVE)
@@ -880,7 +965,7 @@ draw_box (GtkStyle *style,
 	else if (DETAIL ("buttondefault"))
 	{
 		/* draw Default Button marking, but only when sensitive */
-		if (widget && GTK_WIDGET_SENSITIVE (widget))
+		if (widget && state_type != GTK_STATE_INSENSITIVE)
 		{
 			gdk_cairo_set_source_color (cr, &style->bg[GTK_STATE_SELECTED]);
 			cx = x + 0.5; cy = y + 0.5; cw = width - 1.0; ch = height - 1.0;
@@ -915,27 +1000,34 @@ draw_box (GtkStyle *style,
 	else if (DETAIL ("bar"))
 	{
 		GtkProgressBarOrientation orientation;
-		if (widget && GTK_IS_PROGRESS_BAR (widget))
+		gdouble fraction = 0;
+		if (widget && GE_IS_PROGRESS_BAR (widget))
+		{
 			orientation = gtk_progress_bar_get_orientation (GTK_PROGRESS_BAR (widget));
+			fraction = gtk_progress_bar_get_fraction (GTK_PROGRESS_BAR (widget));
+		}
 		else
 			orientation = GTK_PROGRESS_LEFT_TO_RIGHT;
-		paint_progress_bar (cr, style, state_type, orientation, x, y, width, height);
+		paint_progress_bar (cr, style, state_type, orientation, x, y, width, height, fraction);
 	}
-	else if (DETAIL ("trough") && GTK_IS_SCALE (widget))
+	else if (DETAIL ("trough") && GE_IS_SCALE (widget))
 	{
-		if (GTK_IS_VSCALE (widget))
+		GtkOrientation orientation;
+		if (GE_IS_VSCALE (widget))
 		{
 			x += 1; width -= 2;
+			orientation = GTK_ORIENTATION_VERTICAL;
 		}
 		else
 		{
 			y += 1; height -= 2;
+			orientation = GTK_ORIENTATION_HORIZONTAL;
 		}
-		paint_button (cr, style, GTK_STATE_NORMAL, GTK_SHADOW_OUT, x, y, width, height);
+		paint_scale_trough (cr, style, state_type, orientation, x, y, width, height);
 	}
-	else if (DETAIL ("trough") && GTK_IS_SCROLLBAR (widget))
+	else if (DETAIL ("trough") && GE_IS_SCROLLBAR (widget))
 	{
-		if (GTK_IS_HSCROLLBAR (widget))
+		if (GE_IS_HSCROLLBAR (widget))
 			paint_scrollbar_trough (cr, style, state_type, GTK_ORIENTATION_HORIZONTAL, x, y, width, height);
 		else
 			paint_scrollbar_trough (cr, style, state_type, GTK_ORIENTATION_VERTICAL, x, y, width, height);
@@ -945,18 +1037,27 @@ draw_box (GtkStyle *style,
 		/* fill  */
 		cairo_rectangle (cr, x, y, width, height);
 
-		if (DETAIL ("toolbar") || DETAIL ("menubar") || DETAIL ("vscrollbar") || DETAIL ("hscrollbar") || DETAIL ("handlebox") || DETAIL ("dockitem"))
+		if (DETAIL ("toolbar") || DETAIL ("menubar") || DETAIL ("vscrollbar")
+				|| DETAIL ("hscrollbar") || DETAIL ("handlebox") || DETAIL ("dockitem"))
 		{
 			if (shadow_type == GTK_SHADOW_OUT || shadow_type == GTK_SHADOW_ETCHED_OUT)
 			{
 				cairo_pattern_t *crp;
 				CairoColor c1, c2;
 				gdouble shade_v = 0.1;
+				GtkOrientation gradient_orientation = GTK_ORIENTATION_HORIZONTAL;
 
-				crp = cairo_pattern_create_linear (x, y, x, y + height);
 				ge_gdk_color_to_cairo (&style->bg[state_type], &c1);
-				if (DETAIL ("vscrollbar") || DETAIL ("hscrollbar")) 
+				if (DETAIL ("vscrollbar") || DETAIL ("hscrollbar"))
+				{
 					shade_v = 0.3;
+					if (DETAIL ("vscrollbar"))
+						gradient_orientation = GTK_ORIENTATION_VERTICAL;
+				}
+				if (gradient_orientation == GTK_ORIENTATION_HORIZONTAL)
+					crp = cairo_pattern_create_linear (x, y, x, y + height);
+				else
+					crp = cairo_pattern_create_linear (x, y, x + width, y);
 				ge_shade_color (&c1, 1.0 - shade_v, &c2);
 				ge_shade_color (&c1, 1.0 + shade_v, &c1);
 				/*
@@ -981,6 +1082,33 @@ draw_box (GtkStyle *style,
 			gdk_cairo_set_source_color (cr, &style->bg[state_type]);
 			cairo_fill (cr);
 		}
+
+		if (GE_IS_PROGRESS_BAR (widget) && DETAIL ("trough"))
+		{
+			/* draw a shadow in progress bar troughs */
+			gdouble cx, cy, cw, ch;
+			cx = x + 1.5;
+			cy = y + 1.5;
+			cw = width - 3.0;
+			ch = height - 3.0;
+			cairo_move_to (cr, cx + 1.0, cy+ ch - 1.0);
+			cairo_line_to (cr, cx + 1.0, cy+ 1.0);
+			cairo_line_to (cr, cx + cw - 1.0, cy+ 1.0);
+			cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.22);
+			cairo_stroke (cr);
+
+			cairo_move_to (cr, cx + 2.0, cy + ch - 1.0);
+			cairo_line_to (cr, cx + 2.0, cy + 2.0);
+			cairo_line_to (cr, cx + cw - 1.0, cy+ 2.0);
+			cairo_line_to (cr, cx + cw - 1.0, cy+ ch - 1.0);
+			cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.12);
+			cairo_stroke (cr);
+
+			cairo_rectangle (cr, cx + 3.0, cy + 3.0, cw - 5.0, ch - 4.0);
+			cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.03);
+			cairo_stroke (cr);
+		}
+
 		paint_shadow (cr, style, state_type, shadow_type, x, y, width, height);
 	}
 
@@ -1008,63 +1136,37 @@ draw_arrow (GtkStyle *style,
 	if (arrow_type == (GtkArrowType)4)/*NONE - new in GTK 2.10*/
 		return;
 
+
+	/* random adjustments... */
 	if (DETAIL ("vscrollbar") || DETAIL ("hscrollbar"))
 	{
-		/* add some padding */
-		x++; y++; height--; width--; 
-		if (DETAIL ("hscrollbar"))
-			width--;
-		else
-			height--;
-
-		/* random adjustments... */
 		if (arrow_type == GTK_ARROW_DOWN)
 			y++;
 		else if (arrow_type == GTK_ARROW_RIGHT)
 			x++;
-			
 	}
-	else if (DETAIL ("spinbutton"))
+
+	if (DETAIL ("spinbutton"))
 	{
 		if (arrow_type == GTK_ARROW_DOWN)
 			y+=2;
-		else if (arrow_type == GTK_ARROW_UP)
-			y--;
-
-		x++; width--;
-
 	}
-	else if (DETAIL ("menuitem"))
-	{
-		/* make slightly smaller... */
-		x++;
-		height-=2;
-	} else if (ge_is_combo_box_entry (widget) || ge_is_combo (widget) || ge_is_combo_box (widget, FALSE))
-	{
-		x+=2; y+=3; width -= 4; height -= 4;
-		if (ge_is_combo_box (widget, FALSE))
-			x++;
-		if (ge_is_combo_box_entry (widget))
-			x--;
-	}
-	else
-	{
-		/* catch all to add some padding... */
-		x++;
-		y++;
-		width -= 2;
-		height -= 2;
-	}
-
-	/* make triangle equilateral */
 	if (arrow_type == GTK_ARROW_UP || arrow_type == GTK_ARROW_DOWN)
-		height = sin (60.0 * 0.017453) * width;
+	{
+		x = x + width / 2 - 2;
+		y = y + height / 2 - 2;
+		height = 4; width = 5;
+	}
 	else if (arrow_type == GTK_ARROW_LEFT || GTK_ARROW_RIGHT)
-		width = sin (60.0 * 0.017453) * height;
+	{
+		x = x + width / 2 - 2;
+		y = y + height / 2 - 2;
+		height = 5; width = 4;
+	}
+
 
 
 	cr = ge_gdk_drawable_to_cairo (window, area);
-
 	switch (arrow_type)
 	{
 		case GTK_ARROW_UP:
@@ -1128,7 +1230,7 @@ draw_check (GtkStyle *style,
 	ch = height - 1.0; cw = width - 1.0;
 
 
-	if (!GTK_IS_MENU_ITEM (widget))
+	if (!GE_IS_MENU_ITEM (widget))
 	{
 		ge_cairo_rounded_rectangle (cr, cx, cy, cw, ch, 2.0, CR_CORNER_ALL);
 		if (state_type == GTK_STATE_INSENSITIVE)
@@ -1244,9 +1346,9 @@ draw_option (GtkStyle *style,
 	cy = y + (height / 2);
 	radius = (height / 2) - 0.5;
 
-	if (!GTK_IS_MENU_ITEM (widget))
+	if (!GE_IS_MENU_ITEM (widget))
 	{
-		cairo_arc (cr, cx, cy, radius, 0.0, M_PI * 2);
+		cairo_arc (cr, cx, cy, radius, 0.0, G_PI * 2);
 
 		if (state_type == GTK_STATE_INSENSITIVE)
 		{
@@ -1258,15 +1360,15 @@ draw_option (GtkStyle *style,
 		}
 		else
 		{
-			ge_gdk_color_to_cairo (&style->bg[state_type], &c1);
-			ge_shade_color (&c1, 0.7, &c2);
-			ge_shade_color (&c1, 9.9, &c1);
+			ge_gdk_color_to_cairo (&style->bg[state_type], &c2);
+			ge_shade_color (&c2, 0.7, &c2);
+			ge_gdk_color_to_cairo (&style->white, &c1);
 
 			crp = cairo_pattern_create_linear (x, y, x + height, y + height);
 			if (state_type != GTK_STATE_ACTIVE)
 			{
 				/* out shadow */
-				cairo_pattern_add_color_stop_rgb (crp, 0.0, c1.r, c1.g, c1.b);
+				cairo_pattern_add_color_stop_rgb (crp, 0.2, c1.r, c1.g, c1.b);
 				cairo_pattern_add_color_stop_rgb (crp, 1.0, c2.r, c2.g, c2.b);
 			}
 			else
@@ -1282,7 +1384,7 @@ draw_option (GtkStyle *style,
 			cairo_set_source_rgb (cr, OUTLINE_GRAY);
 			cairo_stroke (cr);
 
-			cairo_arc (cr, cx, cy, radius - 1.0, 0.0, M_PI * 2);
+			cairo_arc (cr, cx, cy, radius - 1.0, 0.0, G_PI * 2);
 			cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.5);
 			cairo_stroke (cr);
 		}
@@ -1294,7 +1396,7 @@ draw_option (GtkStyle *style,
 	if (shadow_type == GTK_SHADOW_IN)
 	{
 		/* draw check mark */
-		cairo_arc (cr, cx, cy, radius, 0.0, M_PI * 2);
+		cairo_arc (cr, cx, cy, radius, 0.0, G_PI * 2);
 
 		if (state_type == GTK_STATE_INSENSITIVE)
 		{
@@ -1342,14 +1444,14 @@ draw_tab (GtkStyle *style,
 	/* Draws an option menu tab (the up and down pointing arrows)
 	 * TODO: Make this look neater
 	 */
-    	debug ("draw_tab: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
+	debug ("draw_tab: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
 	    detail, state_type, shadow_type, x, y, width, height);
 
 	draw_arrow (style, window, state_type, shadow_type, area, widget, detail,
-		    GTK_ARROW_UP, FALSE, x + width / 2, y - height / 2, height, height - 1);
+		    GTK_ARROW_UP, FALSE, x + (width / 2) - 2, y - (height / 2), 7, 7);
 
 	draw_arrow (style, window, state_type, shadow_type, area, widget, detail,
-		    GTK_ARROW_DOWN, FALSE, x + width / 2, y + height / 2 + 1, height, height - 1);
+		    GTK_ARROW_DOWN, FALSE, x + (width / 2) - 2, y + (height / 2), 7, 7);
 }
 
 static void
@@ -1460,27 +1562,27 @@ draw_box_gap (GtkStyle *style,
     case GTK_POS_TOP:
 	rect.x = x + gap_x;
 	rect.y = y;
-	rect.width = gap_width;
+	rect.width = gap_width - 1;
 	rect.height = 2;
 	break;
     default:
     case GTK_POS_BOTTOM:
 	rect.x = x + gap_x;
 	rect.y = y + height - 2;
-	rect.width = gap_width;
+	rect.width = gap_width - 1;
 	rect.height = 2;
 	break;
     case GTK_POS_LEFT:
 	rect.x = x;
 	rect.y = y + gap_x;
 	rect.width = 2;
-	rect.height = gap_width;
+	rect.height = gap_width - 1;
 	break;
     case GTK_POS_RIGHT:
 	rect.x = x + width - 2;
 	rect.y = y + gap_x;
 	rect.width = 2;
-	rect.height = gap_width;
+	rect.height = gap_width - 1;
 	break;
     }
 
@@ -1499,32 +1601,17 @@ draw_extension (GtkStyle *style,
 		gint x,
 		gint y, gint width, gint height, GtkPositionType gap_side)
 {
-	GtkWidget* parent_widget;
-	gint relative_x;
-
 	cairo_t *cr;
 	cairo_pattern_t *crp;
 	CairoColor c1, c2;
 	CairoCorners corners;
 
-    debug ("draw_extension: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
-	    detail, state_type, shadow_type, x, y, width, height);
-
-	/* Get x relative to parent widget, not window */
-	if (widget != NULL &&
-	    (parent_widget = gtk_widget_get_parent(widget)) != NULL)
-	{
-		relative_x = x - parent_widget->allocation.x;
-		if (GTK_IS_CONTAINER(widget))
-			relative_x = relative_x - gtk_container_get_border_width((GtkContainer*)widget);
-	}
-	else
-	{
-		relative_x = x;
-	}
+	debug ("draw_extension: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
+		detail, state_type, shadow_type, x, y, width, height);
 
 	cr = ge_gdk_drawable_to_cairo (window, area);
 
+	cairo_save (cr); /* save initial clip for later */
 	cairo_rectangle (cr, x, y, width, height);
 	cairo_clip_preserve (cr);
 	cairo_new_path (cr);
@@ -1536,8 +1623,8 @@ draw_extension (GtkStyle *style,
 	{
 		case GTK_POS_TOP:
 			/* bottom tab */
-			y--;
-			y--;
+			y -= style->ythickness;
+			height += style->ythickness;
 			corners = CR_CORNER_BOTTOMLEFT + CR_CORNER_BOTTOMRIGHT;
 			crp = cairo_pattern_create_linear (x, y + height, x, y);
 			ge_shade_color (&c2, 0.8, &c1);
@@ -1545,22 +1632,20 @@ draw_extension (GtkStyle *style,
 		default:
 		case GTK_POS_BOTTOM:
 			/* top tab */
-			height++;
-			height++;
+			height += style->ythickness;
 			corners = CR_CORNER_TOPLEFT + CR_CORNER_TOPRIGHT;
 			crp = cairo_pattern_create_linear (x, y, x, y + height);
 			break;
 		case GTK_POS_LEFT:
 			/* right tab */
-			x--;
-			x--;
+			x -= style->xthickness;
+			width += style->xthickness;
 			corners = CR_CORNER_TOPRIGHT + CR_CORNER_BOTTOMRIGHT;
 			crp = cairo_pattern_create_linear (x + width, y, x, y);
 			break;
 		case GTK_POS_RIGHT:
 			/* left tab */
-			width++;
-			width++;
+			width += style->xthickness;
 			corners = CR_CORNER_BOTTOMLEFT + CR_CORNER_TOPLEFT;
 			crp = cairo_pattern_create_linear (x, y, x + width, y);
 			break;
@@ -1577,9 +1662,25 @@ draw_extension (GtkStyle *style,
 	cairo_set_source_rgb (cr, OUTLINE_GRAY);
 	cairo_stroke (cr);
 	
+
 	x++; y++; width -= 2.0; height -= 2.0;
+	if (state_type == GTK_STATE_NORMAL)
+	{
+		cairo_restore (cr); /* restore the original clip */
+		switch (gap_side)
+		{
+			case GTK_POS_BOTTOM: height += 2; cairo_rectangle (cr, x, y, width, height -1); break; /* top */
+			case GTK_POS_TOP: y -= 2; height += 2; cairo_rectangle (cr, x, y + 1, width, height - 1); break; /* bottom */
+			case GTK_POS_LEFT: x -= 2; width += 2; cairo_rectangle (cr, x + 1, y, width - 1, height); break; /* right */
+			case GTK_POS_RIGHT: width += 2; cairo_rectangle (cr, x, y, width - 1, height); break; /* left */
+		}
+		
+		cairo_clip_preserve (cr);
+		cairo_new_path (cr);
+	}
+
 	ge_cairo_rounded_rectangle (cr, x + 0.5, y + 0.5, width - 1.0, height - 1.0, 1.0, corners);
-	if (gap_side == GTK_POS_TOP)
+	if (gap_side == GTK_POS_TOP && state_type == GTK_STATE_NORMAL)
 		cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.2);
 	else
 		cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.5); /* must be same colour as corresponding draw_box highlight */
@@ -1627,6 +1728,7 @@ draw_slider (GtkStyle *style,
 	cairo_t *cr;
 	cairo_pattern_t *crp;
 	CairoColor c1, c2;
+	gboolean is_scale = (DETAIL ("vscale") || DETAIL ("hscale"));
 
 	CHECK_ARGS
 	SANITIZE_SIZE
@@ -1635,21 +1737,35 @@ draw_slider (GtkStyle *style,
 
 	cr = ge_gdk_drawable_to_cairo (window, area);
 
-	cairo_rectangle (cr, x + 0.5, y + 0.5, width - 1.0, height - 1.0);
-
-	if (orientation == GTK_ORIENTATION_VERTICAL)
-		crp = cairo_pattern_create_linear (x, y, x + width, y);
-	else
-		crp = cairo_pattern_create_linear (x, y, x, y + height);
 
 	ge_gdk_color_to_cairo (&style->bg[GTK_STATE_SELECTED], &c2);
 
 	/* light */
 	CRUX_LIGHT (c2, c1)
 
+	if (state_type == GTK_STATE_PRELIGHT)
+	{
+		CRUX_LIGHT (c1, c1)
+	}
+
 	/* dark */
 	CRUX_DARK (c2, c2)
 
+	if (widget != NULL && GTK_WIDGET_HAS_FOCUS (widget))
+	{
+		ge_cairo_rounded_rectangle (cr, x+0.5, y+0.5, width-1.0, height-1.0, 2, CR_CORNER_ALL);
+	}
+
+	if (orientation == GTK_ORIENTATION_VERTICAL)
+		crp = cairo_pattern_create_linear (x, y, x + width, y);
+	else
+		crp = cairo_pattern_create_linear (x, y, x, y + height);
+
+	if (is_scale)
+		ge_cairo_rounded_rectangle (cr, x+0.5, y+0.5, width-1.0, height-1.0, 2, CR_CORNER_ALL);
+	else
+		cairo_rectangle (cr, x + 0.5, y + 0.5, width - 1.0, height - 1.0);
+
 	cairo_pattern_add_color_stop_rgb (crp, 0.0, c1.r, c1.g, c1.b);
 	cairo_pattern_add_color_stop_rgb (crp, 1.0, c2.r, c2.g, c2.b);
 	cairo_set_source (cr, crp);
@@ -1661,7 +1777,11 @@ draw_slider (GtkStyle *style,
 
 
 	/* bevel */
-	cairo_rectangle (cr, x+1.5, y+1.5, width-3.0, height-3.0);
+	if (is_scale)
+		ge_cairo_rounded_rectangle (cr, x+1.5, y+1.5, width-3.0, height-3.0, 2, CR_CORNER_ALL);
+	else
+		cairo_rectangle (cr, x+1.5, y+1.5, width-3.0, height-3.0);
+
 	if (orientation == GTK_ORIENTATION_VERTICAL)
 		crp = cairo_pattern_create_linear (x, y, x + width, y);
 	else
@@ -1703,14 +1823,7 @@ draw_handle (GtkStyle *style,
 	state_type = GTK_STATE_ACTIVE;
 
 
-/*
-	cairo_t *cr;
-	cr = ge_gdk_drawable_to_cairo (window, NULL);
-	paint_shadow (cr, style, state_type, GTK_SHADOW_OUT,x, y, width, height);
-	cairo_destroy (cr);
-	*/
-
-	if (!ge_is_panel_widget_item (widget) && !ge_object_is_a (widget, "PanelToplevel"))
+	if (!ge_is_panel_widget_item (widget) && !ge_object_is_a ((GObject*)widget, "PanelToplevel"))
 		draw_box (style, window, state_type, shadow_type, area, widget, detail, x, y, width, height);
 
     light_gc = style->light_gc[state_type];
@@ -1731,7 +1844,7 @@ draw_handle (GtkStyle *style,
     if (height > width)
     {
 	yy = y + height / 2 - 4;
-	for (i = 0; i < 9; i += 2)
+	for (i = 0; i < 8; i += 2)
 	{
 	    gdk_draw_line (window, dark_gc, xthick, yy + i,
 			   x + width - xthick, yy + i);
@@ -1741,8 +1854,8 @@ draw_handle (GtkStyle *style,
     }
     else
     {
-	xx = x + width / 2 - 5;
-	for (i = 0; i < 10; i += 4)
+	xx = x + width / 2 - 4;
+	for (i = 0; i < 8; i += 2)
 	{
 	    gdk_draw_line (window, dark_gc, xx + i, ythick,
 			   xx + i, y + height - ythick);
